#How To

##Installation

###Running on local Node interpreter

Use `node src/testApp.js` to start the service.
Set the environment variable for your needs and add SSL certificate if needed.

###Running on Docker

If new to docker see [Docker Get Started](https://docs.docker.com/get-started/) for your first steps.

This project contains a [Dockerfile](./Dockerfile) to build an image.  
Set the environment variable for your needs and add SSL certificate if needed.

Build the docker image using `$ docker build --tag <yourname>/wrs .` from the projects root directory

Run the docker image using `$ docker run --expose <apiport> --expose <nodeport> --name wrs <yourname>/wrs`.

To use SSL encryption on the WRS Service the SSL certificate and key files need to be passed to the service.
Here we use a docker volume to make the files accessible by the service.
Bind volume for SSL certificate using additional launch parameters:

`--volume <localpath>:/ssl --env WRS_SSL=true --env WRS_SSL_CERT_PATH=/ssl/<fileneme>.cert --env WRS_SSL_KEY_PATH=/ssl/<filename>.key`

**Docker Cluster**

To deploy the service as a cluster in docker use the [docker-compose.yml](./docker-compose.yml).  
Build the image as described above an deploy this service using:
```
$ docker swarm init
$ docker stack deploy -c docker-compose.yml wrs-service
```




###Running on Kubernetes

If new to Kubernetes see [Kubernetes Getting Started](https://kubernetes.io/docs/setup/) and 
[Kubernetes Tutorials](https://kubernetes.io/docs/tutorials/) for your first steps.

---
<span style="color:red">TODO: HOW TO KUBERNETES</span>

---

##Configuration

WRS can be configured to your needs.
The configuration lets you:
* specify SSL certificates to encrypt communications
* secure access this token authentication
* enable/disable the Browser-Only support
* set ports for the API and Node communication
* define your DNS Name used by your service
* set Token and ID length generated by the service

These configurations are set by environment variables.

###Environment Variables

| environment variable             | type     |   range              |    description                                           | default
|:---------------------------------|:---------|:--------------------:|:---------------------------------------------------------|:--------
| WRS_API_PORT                     | {Number} | >= 0 & < 65536       | Port                                                     | 80
| WRS_TOKEN_LENGTH                 | {Number} |  0                   | length of channel token                                  | 16
| WRS_DOMAIN                       | {String} | IP or DNS            | ip or domain use for service (IP or DNS)                 | localhost
| WRS_AUTH                         | {Boolean}|                      | if auth token used for api                               | false
| WRS_AUTH_TOKEN                   | {String} |                      | auth token for api                                       | 
| WRS_AUTH_RESOURCES               | {String} | METOHOD:/path        | list of resources protected by auth: method:path(regex)  | ""
| WRS_SSL                          | {Boolean}|                      | if ssl is used                                           | false
| WRS_SSL_CERT_PATH                | {String} | path                 | path to ssl certificate                                  | 
| WRS_SSL_KEY_PATH                 | {String} | path                 | path to ssl key                                          | 
| WRS_BROWSER_ONLY                 | {Boolean}|                      | activate browser only mode (not restful)                 | false
| WRS_README_PATH                  | {String} | path                 | path to readme for get root                              | 
| WRS_NODE_PORT                    | {Number} | >= 0 & < 65536       | port for raft node communication                         | 4000
| WRS_NODE_HEARTBEAT               | {Number} | > 0                  | raft node heartbeat interval in ms                       | 500 ms
| WRS_NODE_MIN_ELECTION_TIMEOUT    | {Number} | > 0 & > HB           | raft node min election timeout in ms                     | 2000 ms
| WRS_NODE_MAX_ELECTION_TIMEOUT    | {Number} | > 0 & > HB & > MAX   | raft node max election timeout in ms                     | 3000 ms
| WRS_ID_LENGTH                    | {Number} | > 0                  | length of random ids                                     | 4

###Command Line Parameters

the most often used configuration parameters are also accessible by command line Parameters

| key |  parameter | description                               | env. var.
|:----|:-----------|:------------------------------------------|:-------------------
|  -p | --portAPI  | Port to be used for REST-API              | WRS_API_PORT
|  -n | --portNode | Port to be used for Cluster Communication | WRS_NODE_PORT
|  -a | --auth     | Enables token authentication (true/false) | WRS_AUTH
|  -t | --token    | Token to access API                       | WRS_AUTH_TOKEN
|  -s | --ssl      | Enables SSL (true/false)                  | WRS_SSL

command line parameters override the corresponding environment variables for the given process

## Auth

WRS is able to use token authentication for accessing the REST-API

`WRS_AUTH` defines weather this auth token is used at all or not (true odr false).

`WRS_AUTH_TOKEN` sets the token that is used to access the APIs resources. The auth token can be given 
as Authentication Header as Bearer Token, as token field in the body or as url parameter. see README.md for more details

`WRS_AUTH_RESOURCES` is used to define which resources should be locked by token.
If this variable is left undefined or empty and `WRS_AUTH` is set, all resources are locked.
The resources are listed using one whitespace and are described by "method" and "path". The method can be every supported
HTTP Request Method or '*' to select all. The path is a regular expression, that describes the path.

**Examples**

1. `WRS_AUTH_RESOURCES=POST:/channels DELETE:/channels` locks all POST and DELETE requests on the resource "channels"
2. `WRS_AUTH_RESOURCES=*:/cluster` locks all operations on the resource cluster
3. `WRS_AUTH_RESOURCES=POST:/cluster/connect\w+`locks all connection requests

##Connecting Node to Cluster

If a started node has been give an domain via `WRS_DOMAIN` if will try to access the cluster information via this API
using his own domain, his API port and his auth token. If no cluster information is found it will start do work on its
own.  
You can force a connection attempt by sending a POST request to `/cluster/join/:address` providing an ip-address with
`ip:port` or `domain:port`. The node will try to connect, follow redirects and respond on success or failure with status
code and message.

##Disconnecting Node from Cluster

A node disconnects itself from this cluster if its process is ended correctly.

##API usage

For instructions on how to use the WRS API  
see [API_README](README_API.md)


___

Version: 1.0.0  
Last-updated: 19.06.2019