#How to use WRS from your Project

##Basics

This service can be used to exchange messages between clients only using Websocket clients.
The user creates a messaging channel using this REST API and sets a messaging pattern. The API uses HTTP Requests to
create and delete Channels and HTTP Upgrade Request, generated by Websocket Clients to connect.
For more Information on how to use the REST API see [API_README](README_API.md).  

###Parameters

Three main parameters define how a channel works. These are:

**Timeout** _{Number}_  
The timeout sets the timeout after which the channel is deleted.
The timeout is reset by any incoming message.
Timeout is given in milliseconds and must be >0.
A timeout of -1 disables the timeout.  
Default: 86400000 ms (24h)  
minimum: 50 ms (check interval)

**Client Limit** _{Number}_  
The clientLimit limits the count of Clients that con connect to the channel.
All Clients that exceed this limit are rejected.
A clientLimit of -1 disables the Client Limit.  
Default: -1

**Pattern** _{String}_  
The Messaging Patters defines how incoming messages are relayed.
A Messaging Pattern is always mandatory.

###Messaging Patterns

**Echo** ("echo")  
The Echo Pattern simply echos every message back to ist origin.

**Public-Subscribe** ("pub-sub")  
The Public-Subscribe Pattern simply broadcasts each message to all connected clients, including its origin.

**Push-Pull** ("push-pull")
The Push-Pull Pattern distributes messages in a round robin stile, only returning the message to ist origin if no 
other client is available.

**Request-Reply** ("req-repl")
The Request-Reply Pattern distributes Requests to all clients. It the returns a 
response to its origin. For this association a simple JSON-based sub-protocol is needed. for more information
see [API_README](README_API.md)/Messaging Patterns

**Fan-in/out** ("fan-in-out")
This pattern combines the Push-Pull mechanism with the Request-Reply sub-protocol to Fan-out Tasks and to Fan-in
responses.

###Workflow

To use the Service typically this following steps are needed:

+ create channel
  - send HTTP POST Request to `/channels` giving the required pattern
  - save `id` and `token` fields
  - share `id` and `token` with other clients
  
+ connect to channel
  - create an Websocket Client and connect to the API via Websocket url
  - send and receive Messages using this Websocket
  - disconnect
  
+ delete channel
  - send HTTP DELETE Request to `/channel/:id` or wait for the channel to timeout

##Examples

For the following examples we assume our WRS is running on `wrs.net:80` and is has SSL and AUTH disabled.
We want to create a Publish-Subscribe channel for text messaging.

create Channel:
```JavaScript
    var id = "";
    var token = "";
    var xhr = new XMLHttpRequest();
    // create POST Request
    xhr.open("POST", `http//wrs.net:80/channels`, true);
    // set Headers
    xhr.setRequestHeader("Content-Type", "application/json");
    let body = JSON.stringify({
        "pattern": "pub-sub" ,
        "timeout": 3600000,
        "clientLimit": -1
    });
    // save Response
    xhr.onreadystatechange = function()  {
        if (this.readyState !== 4) return;
        let data = JSON.parse(this.responseText);
        id = data.data.id;
        token = data.token;
    };
    // send
    xhr.send(body);
```

connect Websocket:
```Javascript
    let ip = `ws://wrs.net:80/channels/${id}/connect?token=${token}`;
    websocket = new WebSocket(ip);
    // send messages with button
    websocket.onopen = function () {
        myButton.onclick = function () {
            let msg = "some text";
            websocket.send(`${msg}`);
        };
    };
    // print incoming messages
    websocket.onmessage = function (event) {
        console.log(`${event.data}`);
    };
    websocket.onclose = function () {};
```

delete Channel
```JavaScript
    var xhr = new XMLHttpRequest();
    // create DELETE Request
    xhr.open("DELETE", `http//wrs.net:80/channels/${id}`, true);
    xhr.setRequestHeader("Content-Type", "application/json");
    // send
    xhr.send();
```
___

Version: 1.0.0  
Last-updated: 26.08.2019